/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is LightSMS.
 *
 * The Initial Developer of the Original Code is
 * Byungwook Kang.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***********************************************************
constants
***********************************************************/

const nsIAutoCompleteSearch = Components.interfaces.nsIAutoCompleteSearch;

const CLASS_ID = Components.ID("{7cb3298e-2538-11dc-8314-0800200c9a66}");

// description
const CLASS_NAME = "Autocomplete from Text File";

// textual unique identifier
const CONTRACT_ID = "@mozilla.org/autocomplete/search;1?name=text-file";

const CharSet="euc-kr"
/***********************************************************
class definition
***********************************************************/

//class constructor
function TextFile() {
};

// class definition
TextFile.prototype = {
  db:{},
  addEntry: function(name,ndata,front){
    if(!this.db[name]){
      this.db[name]=this.loadFile(name);
    }
    ndata=ndata.replace(/\n/g,"\\n");
    var data=this.db[name];
    var t;
    for each(t in data){
      if(t==ndata)return;
    }
    if(front){
      data.splice(0,0,ndata);
    }else{
      data.push(ndata);
    }
    this.saveFile(name,data);
  },
  removeValue: function(name,str){
    var data=this.db[name];
    var i;
    for(i=0;i<data.length;++i){
      if(data[i]==str)break;
    }
    if(i<data.length){
      data.splice(i,1);
      this.saveFile(name,data);
    }
  },

  startSearch: function(searchString, searchParam, previousResult, listener)
  {
    if(!this.db[searchParam]){
      this.db[searchParam]=this.loadFile(searchParam);
    }
    var data=this.db[searchParam];
    var candidats = new Array();
    var candidats2 = new Array();
    var i;
    for each(i in data) {
      if (i.indexOf(searchString) == 0) {
        candidats.push(i);
      }else candidats2.push(i);
    }
    for each(i in candidats2) {
      if (i.indexOf(searchString) != -1) {
        candidats.push(i);
      }
    }
    
    var result = {
      searchParam: searchParam,      
      QueryInterface: function(aIID) {
        if (aIID.equals(nsIAutoCompleteResult) || aIID.equals(nsISupports))
          return this;
        throw Components.results.NS_NOINTERFACE;
      },
      results: null,
      defaultIndex: 0,
      errorDescription: null,
      matchCount: 0,
      searchResult: Components.interfaces.nsIAutoCompleteResult.RESULT_SUCCESS,
      searchString: searchString,
      getCommentAt: function(index) {},
      getStyleAt: function(index) {},
      getValueAt: function(index) {return this.results[index]},
      removeValueAt: function(rowIndex, removeFromDb) {
        this.addr.removeValue(this.searchParam,this.results[rowIndex]);
        this.results.splice(rowIndex,1);
      }
    }
    result.addr=this;
    result.results = candidats;
    result.matchCount = candidats.length;
    listener.onSearchResult(this, result);
  },
  stopSearch: function() { },

  QueryInterface: function(aIID)
  {
    if (!aIID.equals(nsIAutoCompleteSearch) &&
        !aIID.equals(Components.interfaces.nsITextFile) &&
        !aIID.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

TextFile.prototype.loadFile=function(name){
  var data=[];
  try {
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
                       .getService(Components.interfaces.nsIProperties)
                       .get("ProfD", Components.interfaces.nsIFile);
    file.append(name);
    //if(!file.exists())file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0664);
    var istream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                          .createInstance(Components.interfaces.nsIFileInputStream);
    istream.init(file, 0x01, 0664, 0);
    istream.QueryInterface(Components.interfaces.nsILineInputStream);

    var uniConv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
  									.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    uniConv.charset = CharSet;

    var line = {}, hasmore;
    do {
      hasmore = istream.readLine(line);
      if(line.value){
        //var t=line.value.replace(/\\n/g,"\n");
        var t=line.value;
        data.push(uniConv.ConvertToUnicode(t));
      }
    }while(hasmore);
    istream.close();
  }catch(e){}
  return data;
}
TextFile.prototype.saveFile=function(name,data){
  var file = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ProfD", Components.interfaces.nsIFile);
  file.append(name);
  try{
    var os = Components.classes["@mozilla.org/network/file-output-stream;1"]
                          .createInstance(Components.interfaces.nsIFileOutputStream);
    os.init(file, 0x02 | 0x08 | 0x20, 0664, 0);

    var uniConv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
  									.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    uniConv.charset = CharSet;

    var i;
    for each(i in data) {
      i=uniConv.ConvertFromUnicode(i).replace(/\n/g,"\\n")+"\r\n";
      os.write(i,i.length);
    }
    os.close();
  }catch(e){}
}

/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID.
***********************************************************/
var TextFileFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new TextFile()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var TextFileModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME,
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
  },

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return TextFileFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return TextFileModule; }